import cls from "classnames";
import styles from "./App.module.less";
import { useRef, useMemo, useEffect, useState, CSSProperties } from "react";

const initStyle = {
  flexBasis: "auto",
  WebkitLineClamp: 1000,
  flex: 1,
};

const CENTER_MAX_CONFIG = {
  cardHeight: 892,
  maxShowTableLen: 7,
  recommendLen: 5,
  imageHeight: 252,
};

const SIDE_MAX_CONFIG = {
  cardHeight: 742,
  maxShowTableLen: 6,
  recommendLen: 5,
  imageHeight: 162,
};

export function App({ layout = {}, data = {} }: Record<string, any>) {
  const { avatarPosition = "layout_center" } = layout;
  const recommendRef = useRef(null as HTMLDivElement | null);
  const tableRef = useRef(null as HTMLTableElement | null);
  const containerRef = useRef(null as HTMLDivElement | null);
  const [style, setStyle] = useState({ flexBasis: "auto" } as CSSProperties);
  const [imageUrl, setImageUrl] = useState(data.img);
  const MAX_CONFIG =
    avatarPosition === "layout_center" ? CENTER_MAX_CONFIG : SIDE_MAX_CONFIG;
  const isTextEmpty = useMemo(() => {
    return !data.text?.trim();
  }, [data.text]);

  const isPicEmpty = useMemo(() => {
    return !data.img?.trim();
  }, [data.img]);

  const isTableEmpty = useMemo(() => {
    return !data.tableData?.data?.length;
  }, [data.tableData]);

  const showList = useMemo(() => {
    return data.recommend?.filter((item) => !!item?.trim()) || [];
  }, [data.recommend]);

  useEffect(() => {
    setStyle(initStyle);
    if (!data.text || !containerRef.current) return;
    if (containerRef.current.offsetHeight < MAX_CONFIG.cardHeight) return;
    const tableHeight = tableRef.current
      ? tableRef.current.offsetHeight + 16
      : 0;
    const recommendHeight = recommendRef.current
      ? recommendRef.current.offsetHeight + 16
      : 0;
    const textShowHeight =
      containerRef.current.offsetHeight -
      tableHeight -
      recommendHeight -
      (data.img ? MAX_CONFIG.imageHeight : 0);
    const rows = parseInt(`${(textShowHeight - 16) / 34}`);
    setStyle({
      WebkitLineClamp: rows,
      flexBasis: rows * 34,
      flex: undefined,
    });
  }, [data]);

  useEffect(() => {
    setImageUrl(data.img);
  }, [data.img]);

  const renderRecommend = useMemo(() => {
    if (!showList.length) return null;
    const isShowLong = !!showList.find((item: string) => item.length > 4);
    return (
      <div
        ref={recommendRef}
        className={styles.recommend}
      >
        <ul className={isShowLong ? styles.longList : ""}>
          {showList.map((v: string) => (
            <li>{v.slice(0, MAX_CONFIG.recommendLen)}</li>
          ))}
        </ul>
      </div>
    );
  }, [data.recommend, avatarPosition, showList]);

  if (isTextEmpty && isPicEmpty && isTableEmpty && !showList.length)
    return null;

  return (
    <div
      ref={containerRef}
      className={cls(
        styles.card,
        avatarPosition === "layout_left" && styles.layoutLeft,
        avatarPosition === "layout_right" && styles.layoutRight
      )}
    >
      {!isTextEmpty && (
        <div style={style} className={styles.text}>
          {data.text}
        </div>
      )}
      {!isPicEmpty && (
        <img
          className={styles.img}
          src={imageUrl}
          alt=""
        />
      )}
      {!isTableEmpty && (
        <div ref={tableRef} className={styles.table}>
          <p className={styles.title}>{data.tableData.title}</p>
          <table frame={false}>
            {data.tableData.data.map(
              (v: string[], index: number) =>
                index < MAX_CONFIG.maxShowTableLen && (
                  <tr>
                    <td className={styles.label}>{v[0]}</td>
                    <td className={styles.value}>{v[1]}</td>
                  </tr>
                )
            )}
          </table>
        </div>
      )}
      {renderRecommend}
    </div>
  );
}
